/*
 * ============================================================================
 * (C) Copyright IBM Corporation 2006, 2007.  All Rights Reserved.
 * 
 * You may only copy, modify, and distribute these samples internally.
 * These samples have not been tested under all conditions and are provided
 * to you by IBM without obligation of support of any kind.
 *
 * IBM PROVIDES THESE SAMPLES "AS IS" SUBJECT TO ANY STATUTORY WARRANTIES THAT
 * CANNOT BE EXCLUDED. IBM MAKES NO WARRANTIES OR CONDITIONS, EITHER EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OR
 * CONDITIONS OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND
 * NON-INFRINGEMENT REGARDING THESE SAMPLES OR TECHNICAL SUPPORT, IF ANY.
 * ============================================================================
 */
 
//
//
// File: /app/scripts/startup.groovy
// Author: Dan Jemiolo (danjemiolo)
// 
// This script is a startup handler that will create the application's database 
// and execute the SQL statements found in a given file. You can use it in your 
// own applications by adding the following stanza to /config/zero.config:
//
// [/app/handlers]
// events=start
// handler=startup.groovy
// conditions=[true]
// sqlFile=${your.project.name}/{your.sql.file}
// dbKey={your.database.name}
// dbDir=${your.project.name}/{path.to.database}
//


import zero.data.groovy.Manager;

def checkProperty(property)
{
	if (!zcontains("/event/" + property))
	{
		String message = "Start handler is missing required property '${property}'.";
		logger.SEVERE { message };
		throw new RuntimeException(message);
	}
}

def parseSQL()
{
	logger.FINE { "Parsing SQL file ${event.sqlFile}." }

	def sqlFile = new File(event.sqlFile[]);
	
	def sqlFileContent = sqlFile.getText();
	
	def lines = sqlFileContent.split("\n");
	def buffer = new StringBuffer();
	
	//
	// filter out comments, copyright statements, and CONNECT statement
	//
	lines.each() {
		it = it.trim();
		
		if (it.length() > 0 && !it.startsWith("--") && !it.startsWith("CONNECT") && !it.startsWith("connect"))
			buffer << it << '\n';
	}
	
	return buffer.toString().split(";");
}

def executeSQL(statements)
{
	def database = Manager.create(event.dbKey[]);
	
	logger.FINE { "Ready to execute ${statements.length} statement(s) against database '${event.dbKey}'." }
	
	try
	{
		database.inTransaction() {
			statements.each() {
				it = it.trim();
				
				if (it.length() > 0)
					database.update(it);
			}
		}
	}
	
	catch (Throwable error)
	{
		throw new RuntimeException("Could not complete database transaction: ${error.getMessage()}", error);
	}
}

//
// execution begins here!
//

logger.FINE { "Executing Derby startup handler..." };

checkProperty("dbDir");
checkProperty("dbKey");
checkProperty("sqlFile");

//
// if derby database already exists, don't run the script
//
def databaseDir = new File(event.dbDir[]);
if (!databaseDir.isAbsolute()) {
	databaseDir = new File(config.root[], event.dbDir[]);
}

if (databaseDir.exists())
{
	logger.FINE { "Derby database already exists, skipping table population." }
	return;
}

def statements = parseSQL();
executeSQL(statements);

logger.FINE { "Derby startup handler has finished." }

